# generated by datamodel-codegen:
#   filename:  epdx.schema.json
#   timestamp: 2024-02-16T08:04:39+00:00

from __future__ import annotations

from datetime import datetime
from enum import Enum
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, conint


class ImpactCategory(BaseModel):
    a1a3: Optional[float] = None
    a4: Optional[float] = None
    a5: Optional[float] = None
    b1: Optional[float] = None
    b2: Optional[float] = None
    b3: Optional[float] = None
    b4: Optional[float] = None
    b5: Optional[float] = None
    b6: Optional[float] = None
    b7: Optional[float] = None
    c1: Optional[float] = None
    c2: Optional[float] = None
    c3: Optional[float] = None
    c4: Optional[float] = None
    d: Optional[float] = None


class Source(BaseModel):
    name: str
    url: Optional[str] = None


class Standard(Enum):
    EN15804A1 = 'EN15804A1'
    EN15804A2 = 'EN15804A2'
    UNKNOWN = 'UNKNOWN'


class SubType(Enum):
    Generic = 'Generic'
    Specific = 'Specific'
    Industry = 'Industry'
    Representative = 'Representative'


class Unit(Enum):
    M = 'M'
    M2 = 'M2'
    M3 = 'M3'
    KG = 'KG'
    TONES = 'TONES'
    PCS = 'PCS'
    L = 'L'
    M2R1 = 'M2R1'
    KM = 'KM'
    TONES_KM = 'TONES_KM'
    UNKNOWN = 'UNKNOWN'


class Conversion(BaseModel):
    meta_data: str
    to: Unit
    value: float


class EPD(BaseModel):
    adpe: Optional[ImpactCategory] = None
    adpf: Optional[ImpactCategory] = None
    ap: Optional[ImpactCategory] = None
    comment: Optional[str] = None
    conversions: Optional[List[Conversion]] = None
    cru: Optional[ImpactCategory] = None
    declared_unit: Unit
    eee: Optional[ImpactCategory] = None
    eet: Optional[ImpactCategory] = None
    ep: Optional[ImpactCategory] = None
    format_version: str
    fw: Optional[ImpactCategory] = None
    gwp: Optional[ImpactCategory] = None
    hwd: Optional[ImpactCategory] = None
    id: str
    location: str
    mer: Optional[ImpactCategory] = None
    meta_data: Optional[Dict[str, Any]] = None
    mfr: Optional[ImpactCategory] = None
    name: str
    nhwd: Optional[ImpactCategory] = None
    nrsf: Optional[ImpactCategory] = None
    odp: Optional[ImpactCategory] = None
    penre: Optional[ImpactCategory] = None
    penrm: Optional[ImpactCategory] = None
    penrt: Optional[ImpactCategory] = None
    pere: Optional[ImpactCategory] = None
    perm: Optional[ImpactCategory] = None
    pert: Optional[ImpactCategory] = None
    pocp: Optional[ImpactCategory] = None
    published_date: datetime
    reference_service_life: Optional[conint(ge=0)] = None
    rsf: Optional[ImpactCategory] = None
    rwd: Optional[ImpactCategory] = None
    sm: Optional[ImpactCategory] = None
    source: Optional[Source] = None
    standard: Standard
    subtype: SubType
    valid_until: datetime
    version: str



import pytest

import epdx


def test_parse_ilcd(datafix_dir):
    ilcd_file = datafix_dir / "f63ac879-fa7d-4f91-813e-e816cbdf1927.json"
    epd = epdx.convert_ilcd(ilcd_file.read_text())

    assert isinstance(epd, dict)


def test_parse_empty():
    with pytest.raises(epdx.ParsingException):
        epdx.convert_ilcd("{}")


def test_parse_ilcd_str(datafix_dir):
    ilcd_file = datafix_dir / "f63ac879-fa7d-4f91-813e-e816cbdf1927.json"
    epd = epdx.convert_ilcd(ilcd_file.read_text(), as_type='str')

    assert isinstance(epd, str)


def test_parse_ilcd_pydantic(datafix_dir):
    ilcd_file = datafix_dir / "f63ac879-fa7d-4f91-813e-e816cbdf1927.json"
    epd = epdx.convert_ilcd(ilcd_file.read_text(), as_type='pydantic')

    assert isinstance(epd, epdx.EPD)



import json

from .epdx import *
from .pydantic import *

__doc__ = epdx.__doc__
if hasattr(epdx, "__all__"):
    __all__ = epdx.__all__


def convert_ilcd(data: str | dict, *, as_type: str = "dict"):
    """
    Converts a json formatted ILCD+EPD data into EPDx

    The EPDx data can either be returned as a string, a dict or a Pydantic class.
    """

    if isinstance(data, dict):
        data = json.dumps(data)

    try:
        _epd = epdx._convert_ilcd(data)
    except Exception as err:
        raise ParsingException(err)

    if as_type == "str":
        return _epd
    elif as_type == "dict":
        return json.loads(_epd)
    elif as_type == "pydantic":
        return EPD(**json.loads(_epd))
    else:
        raise NotImplemented("Currently only 'dict', 'str' and 'pydantic' is implemented as_type.")


class ParsingException(Exception):
    ...



from pathlib import Path
import json
import epdx.pydantic

ilcd_file = Path(__file__).parent.parent / "data" / "ilcd.json"

print("EPD as dict")
epd_dict = epdx.convert_ilcd(ilcd_file.read_text())
print(json.dumps(epd_dict, indent=2))

print("\nEPD as Pydantic model")
epd_pydantic = epdx.convert_ilcd(ilcd_file.read_text(), as_type="pydantic")
print(epd_pydantic)

print("\nEPD as string")
epd_str = epdx.convert_ilcd(ilcd_file.read_text(), as_type="str")
print(epd_str)
